@@ -1,5 +1,16 @@
-#ifndef __MAPPINGS_H
-#define __MAPPINGS_H
+//
+// Telescope - graphical task switcher
+//
+// (c) Ilya Skriblovsky, 2010
+// <Ilya.Skriblovsky@gmail.com>
+//
+
+// $Id: Mappings.h 94 2010-06-14 08:54:38Z mitrandir $
+
+// Mappings - handling keyboard bindings
+
+#ifndef __TELESCOPE__MAPPINGS_H
+#define __TELESCOPE__MAPPINGS_H
 
 #include <X11/Xlib.h>
 
@@ -13,8 +24,13 @@ class Mappings
     private:
         static Mappings *_instance;
 
-        LinkedList<Mapping*> _mappings;
+        Display *_dpy;
 
+        LinkedList<Mapping*> _mappings;
+        bool _mappingsLoaded;
+
+
+        void loadMappings();
 
         void parseLine(const char *line);
 
