@@ -1,3 +1,12 @@
+//
+// Telescope - graphical task switcher
+//
+// (c) Ilya Skriblovsky, 2010
+// <Ilya.Skriblovsky@gmail.com>
+//
+
+// $Id: Settings.cpp 102 2010-06-29 13:20:56Z mitrandir $
+
 #include "Settings.h"
 
 #include <stdlib.h>
@@ -22,6 +31,7 @@ Settings::Settings()
 
     _backgroundFilename = 0; // Will be set later if not in config file
     _backgroundMode = Stretched;
+    _backgroundR = _backgroundG = _backgroundB = 0;
 
     _headerLeftFilename = strdup("/usr/share/telescope/header-left.png");
     _headerRightFilename = strdup("/usr/share/telescope/header-right.png");
@@ -34,13 +44,19 @@ Settings::Settings()
 
     _borderWidth = 3;
     _textLeftMargin = 7;
-    _textRightMargin = 25;
+    _textRightMargin = 27;
     _closeButtonXSpan = 40;
     _closeButtonYSpan = 40;
 
-    _fontSize = 24;    
+    _fontSize = 24;
     _textYOffset = -7;
 
+    _disableSelection = false;
+
+    _showDesktopThumbnail = false;
+
+    _showDesktopByIconify = false;
+
 
     // Reading
     FILE *f = fopen(CONFIG_FILE, "r");
@@ -103,6 +119,12 @@ Settings::Settings()
                     else if (strcmp(&line[5], "Cropped\n") == 0)
                         _backgroundMode = Cropped;
                 }
+                else if (strncmp(line, "Red=", 4) == 0)
+                    _backgroundR = atoi(&line[4]) / 256;
+                else if (strncmp(line, "Green=", 6) == 0)
+                    _backgroundG = atoi(&line[6]) / 256;
+                else if (strncmp(line, "Blue=", 5) == 0)
+                    _backgroundB = atoi(&line[5]) / 256;
             }
 
             fclose(homeBgFile);
@@ -295,6 +317,12 @@ void Settings::parseOpt(const char *key, const char *value)
         _fontSize = atoi(value);
     else if (strcmp(key, "text.yOffset") == 0)
         _textYOffset = atoi(value);
+    else if (strcmp(key, "selection.disabled") == 0)
+        _disableSelection = parseBool(value);
+    else if (strcmp(key, "show.desktop.thumbnail") == 0)
+        _showDesktopThumbnail = parseBool(value);
+    else if (strcmp(key, "show.desktop.iconify") == 0)
+        _showDesktopByIconify = parseBool(value);
 }
 
 
@@ -304,8 +332,12 @@ bool Settings::parseBool(const char *value)
         return true;
     if (strcmp(value, "on") == 0)
         return true;
+    if (strcmp(value, "On") == 0)
+        return true;
     if (strcmp(value, "yes") == 0)
         return true;
+    if (strcmp(value, "Yes") == 0)
+        return true;
 
     return false;
 }
