@@ -1,5 +1,16 @@
-#ifndef __SETTINGS_H
-#define __SETTINGS_H
+//
+// Telescope - graphical task switcher
+//
+// (c) Ilya Skriblovsky, 2010
+// <Ilya.Skriblovsky@gmail.com>
+//
+
+// $Id: Settings.h 102 2010-06-29 13:20:56Z mitrandir $
+
+// Settings - loads and provides settings from configuration file
+
+#ifndef __TELESCOPE__SETTINGS_H
+#define __TELESCOPE__SETTINGS_H
 
 class Settings
 {
@@ -18,6 +29,7 @@ class Settings
         bool _scrollingEnabled;
         char *_backgroundFilename;
         BackgroundMode _backgroundMode;
+        int _backgroundR, _backgroundG, _backgroundB;
 
         char *_headerLeftFilename;
         char *_headerRightFilename;
@@ -38,6 +50,13 @@ class Settings
         int _textYOffset;
 
 
+        bool _disableSelection;
+
+        bool _showDesktopThumbnail;
+
+        bool _showDesktopByIconify;
+
+
         void parseLine(const char *line);
         void parseOpt(const char *key, const char *value);
 
@@ -55,6 +74,9 @@ class Settings
 
         const char* backgroundFilename() { return _backgroundFilename; }
         BackgroundMode backgroundMode() { return _backgroundMode; }
+        int backgroundR() { return _backgroundR; }
+        int backgroundG() { return _backgroundG; }
+        int backgroundB() { return _backgroundB; }
 
         const char* headerLeftFilename() { return _headerLeftFilename; }
         const char* headerRightFilename() { return _headerRightFilename; }
@@ -73,6 +95,12 @@ class Settings
 
         int fontSize() { return _fontSize; }
         int textYOffset() { return _textYOffset; }
+
+        bool disableSelection() { return _disableSelection; }
+
+        bool showDesktopThumbnail() { return _showDesktopThumbnail; }
+
+        bool showDesktopByIconify() { return _showDesktopByIconify; }
 };
 
 #endif
