@@ -1,17 +1,26 @@
+//
+// Telescope - graphical task switcher
+//
+// (c) Ilya Skriblovsky, 2010
+// <Ilya.Skriblovsky@gmail.com>
+//
+
+// $Id: TeleWindow.cpp 102 2010-06-29 13:20:56Z mitrandir $
+
 #include "TeleWindow.h"
 
 #include <string.h>
 #include <stdlib.h>
+#include <math.h>
 
 #include <sys/time.h>
 
+#include <X11/Xatom.h>
+
 #include "XTools.h"
 #include "Thumbnail.h"
 #include "Settings.h"
-
-
-const int TeleWindow::XMargin = 10;
-const int TeleWindow::YMargin = 15;
+#include "Resources.h"
 
 
 TeleWindow::TeleWindow(Display *dpy)
@@ -40,70 +49,6 @@ TeleWindow::TeleWindow(Display *dpy)
 
 
 
-// Loading header images
-
-    bool ok = true;
-    ok = ok && loadARGBImage(
-        Settings::instance()->headerLeftFilename(),
-        _rootWindow,
-        &_headerLeftPix,
-        &_headerLeftPict,
-        &_headerLeftWidth,
-        &_headerHeight);
-    ok = ok && loadARGBImage(
-        Settings::instance()->headerRightFilename(),
-        _rootWindow,
-        &_headerRightPix,
-        &_headerRightPict,
-        &_headerRightWidth,
-        0);
-    ok = ok && loadARGBImage(
-        Settings::instance()->headerMiddleFilename(),
-        _rootWindow,
-        &_headerMiddlePix,
-        &_headerMiddlePict,
-        0, 0);
-
-    ok = ok && loadARGBImage(
-        Settings::instance()->headerLeftSelectedFilename(),
-        _rootWindow,
-        &_headerLeftSelectedPix,
-        &_headerLeftSelectedPict,
-        0, 0);
-    ok = ok && loadARGBImage(
-        Settings::instance()->headerRightSelectedFilename(),
-        _rootWindow,
-        &_headerRightSelectedPix,
-        &_headerRightSelectedPict,
-        0, 0);
-    ok = ok && loadARGBImage(
-        Settings::instance()->headerMiddleSelectedFilename(),
-        _rootWindow,
-        &_headerMiddleSelectedPix,
-        &_headerMiddleSelectedPict,
-        0, 0);
-
-    if (! ok)
-        printf("Cannot load all pixmaps\n");
-
-    XRenderPictureAttributes pictureAttrs;
-    pictureAttrs.repeat = RepeatNormal;
-    XRenderChangePicture(_dpy, _headerMiddlePict, CPRepeat, &pictureAttrs);
-    XRenderChangePicture(_dpy, _headerMiddleSelectedPict, CPRepeat, &pictureAttrs);
-
-
-    XRenderParseColor(_dpy, Settings::instance()->borderColor(), &_borderColor);
-    XRenderParseColor(_dpy, Settings::instance()->borderActiveColor(), &_borderActiveColor);
-
-
-    _borderWidth = Settings::instance()->borderWidth();
-    _textLeftMargin = Settings::instance()->textLeftMargin();
-    _textRightMargin = Settings::instance()->textRightMargin();
-
-    _closeButtonXSpan = Settings::instance()->closeButtonXSpan();
-    _closeButtonYSpan = Settings::instance()->closeButtonYSpan();
-
-
 
     // Initialization of scrolling
 
@@ -127,6 +72,7 @@ TeleWindow::TeleWindow(Display *dpy)
         &createAttrs);
 
     _gc = XCreateGC(_dpy, _win, 0, 0);
+    XSetGraphicsExposures(_dpy, _gc, false);
 
     XSelectInput(_dpy, _win,
         ExposureMask        |
@@ -139,6 +85,15 @@ TeleWindow::TeleWindow(Display *dpy)
     );
 
 
+    // Setting Window type as Splash. This tells window manager
+    // to not try to decorate our window.
+    Atom windowType = XTools::_NET_WM_WINDOW_TYPE_SPLASH;
+    XChangeProperty(_dpy, _win, XTools::_NET_WM_WINDOW_TYPE,
+        XA_ATOM, 32, PropModeReplace,
+        (const unsigned char*)&windowType, 1);
+
+
+
     // Initializing Xft
 
     char fontname[50];
@@ -153,17 +108,13 @@ TeleWindow::TeleWindow(Display *dpy)
 
     // Double buffering pixmap
     _bufferPix = 0;
-    _xrenderFormat = 0;
     recreateBufferPixmap();
 
 
     XRenderPictureAttributes pa;
     pa.subwindow_mode = IncludeInferiors;
 
-    _picture = XRenderCreatePicture(_dpy, _win, _xrenderFormat, CPSubwindowMode, &pa);
-
-    // Xft Draw object
-    recreateXftDraw();
+    _picture = XRenderCreatePicture(_dpy, _win, XTools::xrenderFormat(), CPSubwindowMode, &pa);
 
 
     _shown = false;
@@ -191,7 +142,6 @@ TeleWindow::~TeleWindow()
         delete *i;
 
 
-    XftDrawDestroy(_xftDraw);
 
     XRenderFreePicture(_dpy, _picture);
     XRenderFreePicture(_dpy, _bufferPicture);
@@ -200,14 +150,6 @@ TeleWindow::~TeleWindow()
     XFreePixmap(_dpy, _bufferPix);
 
 
-    XRenderFreePicture(_dpy, _headerLeftPict);
-    XRenderFreePicture(_dpy, _headerRightPict);
-    XRenderFreePicture(_dpy, _headerMiddlePict);
-    XFreePixmap(_dpy, _headerLeftPix);
-    XFreePixmap(_dpy, _headerRightPix);
-    XFreePixmap(_dpy, _headerMiddlePix);
-
-
     XFreeGC(_dpy, _gc);
     XDestroyWindow(_dpy, _win);
 }
@@ -229,34 +171,45 @@ void TeleWindow::show()
 {
     _shown = true;
 
+
     updateThumbnailsList();
 
     Window activeWindow = XTools::activeWindow();
 
+    Thumbnail *prevActiveThumbnail = _activeThumbnail;
     _activeThumbnail = 0;
     for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i)
         if ((*i)->clientWindow() == activeWindow)
             _activeThumbnail = *i;
 
+    if (_activeThumbnail != prevActiveThumbnail)
+    {
+        if (_activeThumbnail) _activeThumbnail->redraw();
+        if (prevActiveThumbnail) prevActiveThumbnail->redraw();
+    }
+
 
     XMapWindow(_dpy, _win);
 
-    XEvent event;
-
-    memset(&event, 0, sizeof(event));
-    event.type = ClientMessage;
-    event.xclient.window = _win;
-    event.xclient.message_type = XTools::_NET_WM_STATE;
-    event.xclient.format = 32;
-    event.xclient.data.l[0] = 1;
-    event.xclient.data.l[1] = XTools::_NET_WM_STATE_FULLSCREEN;
-    event.xclient.data.l[2] = 0;
-
-
-    XSendEvent(_dpy, _rootWindow, False, SubstructureNotifyMask, &event);
 
     XTools::switchToWindow(_win);
 
+#ifdef MAEMO4
+    // Hack for Nokia's builtin mediaplayer that leaves it's
+    // video overlay on screen after task switching when
+    // Composite is enables
+    if (_activeThumbnail && _activeThumbnail->isOssoMediaPlayer())
+    {
+        paint();
+        XSync(_dpy, 0);
+        struct timeval timeout;
+        timeout.tv_sec = 0;
+        timeout.tv_usec = 100000;
+        select(0, 0, 0, 0, &timeout);
+        _activeThumbnail->minimize();
+    }
+#endif
+
 
 //    if (_activeThumbnail)
 //        animate(_activeThumbnail, true);
@@ -291,10 +244,13 @@ void TeleWindow::updateThumbnailsList()
     if (_thumbnailsListDirty == false)
         return;
 
+
+
     bool wasChanged = false;
 
 
-    LinkedList<Window> windows = XTools::windowList(_rootWindow);
+    LinkedList<Window> windows = XTools::windowList(_rootWindow,
+        Settings::instance()->showDesktopThumbnail());
     for (LinkedList<Window>::Iter i = windows.head(); i; ++i)
         if (*i != _win)
         {
@@ -350,18 +306,17 @@ void TeleWindow::layoutThumbnails()
         return;
     }
 
+
     int maxSize = 0;
     int maxColumns = 0;
 
-    for (int columns = 1; columns <= n; ++columns)
+    for (int columns = n; columns >= 1; --columns)
     {
         int rows = (n + columns - 1) / columns;
 
-        int widthWithBorder = _width / columns - 2 * XMargin;
-        int heightWithBorder = _height / rows - 2 * YMargin;
-
-        int width = widthWithBorder - 2 * _borderWidth;
-        int height = heightWithBorder - _headerHeight - _borderWidth;
+        int border = (int)(_width / columns * 0.1f);
+        int width = _width / columns - border;
+        int height = _height / rows - border;
 
         int resultSize = height * _width / _height;
         if (width < resultSize)
@@ -379,26 +334,23 @@ void TeleWindow::layoutThumbnails()
 
     int tileWidth = _width / columns;
     int tileHeight = _height / rows;
+    int border = (int)(_width / columns * 0.1f);
 
     int thumbWidth, thumbHeight;
 
     if (n > 1)
     {
-        thumbWidth = maxSize;
-        thumbHeight= maxSize * _height / _width;
+        thumbWidth = _width / columns - border;
+        thumbHeight = _height / rows - border;
     }
     else
     {
-        // If there is only one window we don't want it become to big
         thumbWidth = _width * 2 / 3;
         thumbHeight = _height * 2 / 3;
     }
 
-    int widthWithBorder = thumbWidth + 2*_borderWidth;
-    int heightWithBorder= thumbHeight+ _borderWidth + _headerHeight;
-
-    int tileXOffset = (tileWidth - widthWithBorder) / 2;
-    int tileYOffset = (tileHeight-heightWithBorder) / 2;
+    int tileXOffset = (tileWidth - thumbWidth) / 2;
+    int tileYOffset = (tileHeight-thumbHeight) / 2;
 
     int lastRowX = 0;
 
@@ -420,10 +372,10 @@ void TeleWindow::layoutThumbnails()
 
             int tileX = lastRowX + column * tileWidth;
 
-            int x = tileX + tileXOffset + _borderWidth;
-            int y = tileY + tileYOffset + _headerHeight;
+            int x = tileX + tileXOffset;
+            int y = tileY + tileYOffset;
 
-            (*thumb)->setGeometry(x, y, thumbWidth, thumbHeight);
+            (*thumb)->fitIn(x, y, thumbWidth, thumbHeight);
         }
     }
 
@@ -435,6 +387,8 @@ void TeleWindow::layoutThumbnails()
 void TeleWindow::removeThumbnail(Thumbnail *thumb)
 {
     _thumbnails.removeByValue(thumb);
+    if (_activeThumbnail == thumb)
+        _activeThumbnail = 0;
 }
 
 
@@ -518,7 +472,11 @@ void TeleWindow::onRootEvent(XEvent *event)
                 hide();
 
         if (event->xproperty.atom == XTools::_NET_CLIENT_LIST)
+        {
+            // FIXME: This notify seems to be sent too often,
+            // even when no windows actually changed
             markThumbnailsListDirty();
+        }
     }
     else if (event->type == ConfigureNotify)
     {
@@ -575,115 +533,14 @@ void TeleWindow::paint()
 
     for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i)
     {
-        int x = (*i)->x();
-        int y = (*i)->y();
-        int w = (*i)->realWidth();
-        int h = (*i)->realHeight();
-        int xoffset = ((*i)->width() - w) / 2;
-        int yoffset = ((*i)->height() - h) / 2;
-
-        bool selected = (*i) == _activeThumbnail;
-        XRenderColor borderColor = selected ? _borderActiveColor : _borderColor;
-
-        Picture left = selected ? _headerLeftSelectedPict : _headerLeftPict;
-        Picture right = selected ? _headerRightSelectedPict : _headerRightPict;
-        Picture middle = selected ? _headerMiddleSelectedPict : _headerMiddlePict;
-
-        XRenderComposite(_dpy, PictOpOver,
-            left, None, _bufferPicture,
-            0, 0,
-            0, 0,
-            _scrollX + x + xoffset - _borderWidth,
-            _scrollY + y + yoffset - _headerHeight,
-            _headerLeftWidth, _headerHeight
-        );
-
-        XRenderComposite(_dpy, PictOpOver,
-            right, None, _bufferPicture,
-            0, 0,
-            0, 0,
-            _scrollX + x + xoffset + w + _borderWidth - _headerRightWidth,
-            _scrollY + y + yoffset - _headerHeight,
-            _headerRightWidth, _headerHeight
-        );
-
-        XRenderComposite(_dpy, PictOpSrc,
-            middle, None, _bufferPicture,
-            0, 0,
-            0, 0,
-            _scrollX + x + xoffset - _borderWidth + _headerLeftWidth,
-            _scrollY + y + yoffset - _headerHeight,
-            w + 2 * _borderWidth - _headerLeftWidth - _headerRightWidth,
-            _headerHeight
-        );
-
-        // Left border
-        XRenderFillRectangle(_dpy, PictOpSrc, _bufferPicture, &borderColor,
-            _scrollX + x + xoffset - _borderWidth,
-            _scrollY + y + yoffset,
-            _borderWidth, h
-        );
-        // Right border
-        XRenderFillRectangle(_dpy, PictOpSrc, _bufferPicture, &borderColor,
-            _scrollX + x + xoffset + w,
-            _scrollY + y + yoffset,
-            _borderWidth, h
-        );
-        // Bottom border
-        XRenderFillRectangle(_dpy, PictOpSrc, _bufferPicture, &borderColor,
-            _scrollX + x + xoffset - _borderWidth,
-            _scrollY + y + yoffset + h,
-            w + 2*_borderWidth, _borderWidth
-        );
+        (*i)->drawPreview();
+        blitThumb(*i);
     }
 
 
-    XftColor fontColor;
-    fontColor.pixel = 0;
-    fontColor.color.red     = 0xffff;
-    fontColor.color.green   = 0xffff;
-    fontColor.color.blue    = 0xffff;
-    fontColor.color.alpha   = 0xffff;
+    blitBuffer();
 
 
-    for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i)
-    {
-        int xoffset = ((*i)->width() - (*i)->realWidth()) / 2;
-        int yoffset = ((*i)->height() - (*i)->realHeight()) / 2;
-
-        XRectangle rect = {
-            _scrollX + (*i)->x() + xoffset - _borderWidth + _textLeftMargin,
-            _scrollY + (*i)->y() + yoffset - _headerHeight,
-            (*i)->realWidth() + 2*_borderWidth - _textLeftMargin - _textRightMargin,
-            _headerHeight
-        };
-        Region clip = XCreateRegion();
-        XUnionRectWithRegion(&rect, clip, clip);
-
-        XftDrawSetClip(_xftDraw, clip);
-
-        XftDrawStringUtf8(_xftDraw, &fontColor, _xftFont, 
-            _scrollX + (*i)->x() + xoffset - _borderWidth + _textLeftMargin,
-            _scrollY + (*i)->y() + yoffset + Settings::instance()->textYOffset(),
-            (const FcChar8*)(*i)->title(),
-            strlen((*i)->title())
-        );
-
-        XDestroyRegion(clip);
-    }
-
-    for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i)
-        (*i)->paint();
-
-
-
-
-
-
-    XCopyArea(_dpy, _bufferPix, _win, _gc,
-        0, 0, _width, _height, 0, 0
-    );
-
 //    timeval after;
 //    gettimeofday(&after, 0);
 //    printf("---------\n");
@@ -691,14 +548,30 @@ void TeleWindow::paint()
 //    printf("%ld:%ld\n", after.tv_sec, after.tv_usec);
 }
 
+void TeleWindow::blitThumb(Thumbnail *thumb)
+{
+    XRenderComposite(_dpy, PictOpOver,
+        thumb->picture(), None, _bufferPicture,
+        0, 0,
+        0, 0,
+        thumb->x(), thumb->y(),
+        thumb->width(), thumb->height()
+    );
+}
+
+
+void TeleWindow::blitBuffer()
+{
+    XCopyArea(_dpy, _bufferPix, _win, _gc,
+        0, 0, _width, _height, 0, 0
+    );
+}
+
 
 void TeleWindow::onThumbRedrawed(Thumbnail *thumb)
 {
-    XCopyArea(_dpy, _bufferPix, _win, _gc,
-        _scrollX + thumb->x(), _scrollY + thumb->y(),
-        thumb->width(), thumb->height(),
-        _scrollX + thumb->x(), _scrollY + thumb->y()
-    );
+    blitThumb(thumb);
+    blitBuffer();
 }
 
 
@@ -709,16 +582,20 @@ void TeleWindow::onHotKeyPress()
     {
         if (_shown)
             hide();
-        XTools::showDesktop();
+        showDesktop();
         return;
     }
 
     _hotKeyPressed = true;
 
     if (! _shown)
+    {
         show();
+    }
     else
+    {
         _mappings.handleEvent(this, Mapping::Press, _hotKeyCode);
+    }
 }
 
 void TeleWindow::onHotKeyRelease()
@@ -752,37 +629,16 @@ void TeleWindow::onButtonRelease(XButtonEvent *event)
         bool missed = true;
 
         for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i)
-        {
-            // Checking close button
+            if ((*i)->handleMousePress(event->x, event->y))
             {
-                int closeX = (*i)->x() + (*i)->width() + _borderWidth - _closeButtonXSpan;
-                int closeY = (*i)->y() - _headerHeight;
-
-
-                if (- _scrollX + event->x >= closeX &&
-                    - _scrollX + event->x <= closeX + _closeButtonXSpan &&
-                    - _scrollY + event->y >= closeY &&
-                    - _scrollY + event->y <= closeY + _closeButtonYSpan
-                    )
-                {
-                    (*i)->closeClient();
-                    missed = false;
-                }
-            }
-
-            if (missed && (*i)->inside(- _scrollX + event->x, - _scrollY + event->y))
-            {
-//                animate(*i, false);
-                (*i)->switchToClient();
-                hide();
                 missed = false;
+                break;
             }
-        }
 
         if (missed)
         {
             hide();
-            XTools::showDesktop();
+            showDesktop();
         }
     }
 }
@@ -823,67 +679,113 @@ void TeleWindow::internalCommand(const char *action)
 {
     if (strcmp(action, "switchToSelected") == 0)
     {
-        if (_activeThumbnail != 0)
-            _activeThumbnail->switchToClient();
-        else
-            XTools::showDesktop();
-        hide();
+        if (! Settings::instance()->disableSelection())
+        {
+            if (_activeThumbnail != 0)
+                _activeThumbnail->switchToClient();
+            else
+                showDesktop();
+            hide();
+        }
     }
     else if (strcmp(action, "selectNext") == 0)
     {
-        if (_activeThumbnail == 0)
+        if (! Settings::instance()->disableSelection())
         {
-            if (_thumbnails.size() > 0)
-                _activeThumbnail = *_thumbnails.head();
-        }
-        else
-        {
-            int index = 0;
-            bool changed = false;
-            for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i, ++index)
-                if ((*i) == _activeThumbnail)
-                {
-                    if (index < _thumbnails.size() - 1)
-                        _activeThumbnail = *(++i);
-                    else
-                        _activeThumbnail = *_thumbnails.head();
-                    changed = true;
-                    break;
-                }
+            Thumbnail *prevActiveThumbnail = _activeThumbnail;
+            if (_activeThumbnail == 0)
+            {
+                if (_thumbnails.size() > 0)
+                    _activeThumbnail = *_thumbnails.head();
+            }
+            else
+            {
+                int index = 0;
+                bool changed = false;
+                for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i, ++index)
+                    if ((*i) == _activeThumbnail)
+                    {
+                        if (index < _thumbnails.size() - 1)
+                            _activeThumbnail = *(++i);
+                        else
+                            _activeThumbnail = *_thumbnails.head();
+                        changed = true;
+                        break;
+                    }
 
-            if (! changed)
-                _activeThumbnail = *_thumbnails.head();
-        }
+                if (! changed)
+                    _activeThumbnail = *_thumbnails.head();
+            }
 
-        paint();
+            if (_activeThumbnail) _activeThumbnail->redraw();
+            if (prevActiveThumbnail) prevActiveThumbnail->redraw();
+
+            paint();
+        }
     }
     else if (strcmp(action, "selectPrev") == 0)
     {
-        if (_activeThumbnail == 0)
+        if (! Settings::instance()->disableSelection())
         {
-            if (_thumbnails.size() > 0)
-                _activeThumbnail = *_thumbnails.tail();
+            Thumbnail *prevActiveThumbnail = _activeThumbnail;
+            if (_activeThumbnail == 0)
+            {
+                if (_thumbnails.size() > 0)
+                    _activeThumbnail = *_thumbnails.tail();
+            }
+            else
+            {
+                int index = 0;
+                bool changed = false;
+                for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i, ++index)
+                    if ((*i) == _activeThumbnail)
+                    {
+                        if (index > 0)
+                            _activeThumbnail = *(--i);
+                        else
+                            _activeThumbnail = *_thumbnails.tail();
+                        changed = true;
+                        break;
+                    }
+
+                if (! changed)
+                    _activeThumbnail = *_thumbnails.tail();
+            }
+
+            if (_activeThumbnail) _activeThumbnail->redraw();
+            if (prevActiveThumbnail) prevActiveThumbnail->redraw();
+
+            paint();
         }
-        else
+    }
+    else if (   strcmp(action, "selectRight") == 0
+             || strcmp(action, "selectLeft") == 0
+             || strcmp(action, "selectDown") == 0
+             || strcmp(action, "selectUp") == 0
+            )
+    {
+        if (_activeThumbnail != 0 && ! Settings::instance()->disableSelection())
         {
-            int index = 0;
-            bool changed = false;
-            for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i, ++index)
-                if ((*i) == _activeThumbnail)
-                {
-                    if (index > 0)
-                        _activeThumbnail = *(--i);
-                    else
-                        _activeThumbnail = *_thumbnails.tail();
-                    changed = true;
-                    break;
-                }
+            Thumbnail *prevActiveThumbnail = _activeThumbnail;
 
-            if (! changed)
-                _activeThumbnail = *_thumbnails.tail();
+            Thumbnail *newThumbnail = 0;
+            if (strcmp(action, "selectRight") == 0)
+                newThumbnail = findThumbnailByCoords(_activeThumbnail, 1);
+            if (strcmp(action, "selectLeft") == 0)                      
+                newThumbnail = findThumbnailByCoords(_activeThumbnail, 3);
+            if (strcmp(action, "selectUp") == 0)                        
+                newThumbnail = findThumbnailByCoords(_activeThumbnail, 0);
+            if (strcmp(action, "selectDown") == 0)                      
+                newThumbnail = findThumbnailByCoords(_activeThumbnail, 2);
+
+            if (newThumbnail)
+                _activeThumbnail = newThumbnail;
+
+            if (_activeThumbnail) _activeThumbnail->redraw();
+            if (prevActiveThumbnail) prevActiveThumbnail->redraw();
+
+            paint();
         }
-
-        paint();
     }
     else
     {
@@ -892,11 +794,45 @@ void TeleWindow::internalCommand(const char *action)
 }
 
 
+Thumbnail* TeleWindow::findThumbnailByCoords(
+    Thumbnail *orig,
+    int direction
+)
+{
+    Thumbnail *foundThumb = 0;
+    int max = INT_MAX;
+
+    for (LinkedList<Thumbnail*>::Iter i = _thumbnails.head(); i; ++i)
+    {
+        Thumbnail *thumb = *i;
+
+        if (thumb == orig)
+            continue;
+
+        int rx = thumb->x() - orig->x();
+        int ry = thumb->y() - orig->y();
+        int arx = abs(rx);
+        int ary = abs(ry);
+
+        if (ry < 0 && ary > arx && direction != 0) continue;
+        if (rx > 0 && arx > ary && direction != 1) continue;
+        if (ry > 0 && ary > arx && direction != 2) continue;
+        if (rx < 0 && arx > ary && direction != 3) continue;
+
+        if (arx + ary < max)
+        {
+            foundThumb = thumb;
+            max = arx + ary;
+        }
+    }
+
+    return foundThumb;
+}
+
+
 
 void TeleWindow::setNewSize(int width, int height)
 {
-    printf("setNewSize(%d, %d)\n", width, height);
-
     _width = width;
     _height = height;
 
@@ -906,8 +842,6 @@ void TeleWindow::setNewSize(int width, int height)
 
     reloadBackground();
 
-    recreateXftDraw();
-
     layoutThumbnails();
 }
 
@@ -924,16 +858,30 @@ void TeleWindow::reloadBackground()
 
     printf("Loading background from '%s'\n", Settings::instance()->backgroundFilename());
     printf("Background mode: %d\n", Settings::instance()->backgroundMode());
+
+    int bgXpos = 0;
+    int bgYpos = 0;
     Imlib_Image background = imlib_load_image(Settings::instance()->backgroundFilename());
     if (background == 0)
+    {
         printf("Cannot load background\n");
+        printf("rgb = (%d, %d, %d)\n",
+            Settings::instance()->backgroundR(),
+            Settings::instance()->backgroundG(),
+            Settings::instance()->backgroundB());
+        background = imlib_create_image(_width, _height);
+        imlib_context_set_image(background);
+        imlib_context_set_color(
+            Settings::instance()->backgroundR(),
+            Settings::instance()->backgroundG(),
+            Settings::instance()->backgroundB(),
+            255);
+        imlib_image_fill_rectangle(0, 0, _width, _height);
+    }
     else
     {
         Imlib_Image bgToDraw = 0;
 
-        int bgXpos = 0;
-        int bgYpos = 0;
-
         imlib_context_set_image(background);
         switch (Settings::instance()->backgroundMode())
         {
@@ -1018,14 +966,14 @@ void TeleWindow::reloadBackground()
         imlib_free_image();
 
         imlib_context_set_image(bgToDraw);
-        imlib_context_set_drawable(_bgPixmap);
-        imlib_context_set_visual(DefaultVisual(_dpy, DefaultScreen(_dpy)));
-
-
-        imlib_render_image_on_drawable(bgXpos, bgYpos);
-
-        imlib_free_image();
     }
+
+    imlib_context_set_drawable(_bgPixmap);
+    imlib_context_set_visual(DefaultVisual(_dpy, DefaultScreen(_dpy)));
+
+    imlib_render_image_on_drawable(bgXpos, bgYpos);
+
+    imlib_free_image();
 }
 
 
@@ -1039,110 +987,13 @@ void TeleWindow::recreateBufferPixmap()
 
     int scr = DefaultScreen(_dpy);
     int depth = DefaultDepth(_dpy, scr);
-    Visual *visual = DefaultVisual(_dpy, scr);
 
-    if (_xrenderFormat == 0)
-    {
-        _xrenderFormat = XRenderFindVisualFormat(_dpy, visual);
-        XVisualInfo rgbaVisual;
-        if (XMatchVisualInfo(_dpy, scr, 32, TrueColor, &rgbaVisual) == 0)
-            fprintf(stderr, "Cannot find rgba visual\n");
-        _xrenderRGBAFormat = XRenderFindVisualFormat(_dpy, rgbaVisual.visual);
-    }
 
     // Double-buffering pixmap
     _bufferPix = XCreatePixmap(_dpy, _rootWindow, _width, _height, depth);
 
     // XRender picture
-    _bufferPicture = XRenderCreatePicture(_dpy, _bufferPix, _xrenderFormat, 0, 0);
-}
-
-void TeleWindow::recreateXftDraw()
-{
-    if (_xftDraw)
-    {
-        XftDrawDestroy(_xftDraw);
-        _xftDraw = 0;
-    }
-
-    int scr = DefaultScreen(_dpy);
-    Visual *visual = DefaultVisual(_dpy, scr);
-    Colormap colormap = DefaultColormap(_dpy, scr);
-
-    _xftDraw = XftDrawCreate(_dpy, _bufferPix, visual, colormap);
-}
-
-
-bool TeleWindow::loadARGBImage(
-    const char *filename,
-    Drawable parent,
-    Pixmap *pixmap,
-    Picture *picture,
-    int *width,
-    int *height)
-{
-    static Visual *defaultVisual = 0;
-    static XVisualInfo rgbaVisual;
-    static XRenderPictFormat *defaultFormat;
-    static XRenderPictFormat *rgbaFormat;
-    static Colormap colormap;
-
-    *pixmap = 0;
-    *picture = 0;
-
-    if (defaultVisual == 0)
-    {
-        // Initializing
-
-        int scr = DefaultScreen(_dpy);
-        defaultVisual = DefaultVisual(_dpy, scr);
-        colormap = DefaultColormap(_dpy, scr);
-        defaultFormat = XRenderFindVisualFormat(_dpy, defaultVisual);
-
-        if (XMatchVisualInfo(_dpy, scr, 32, TrueColor, &rgbaVisual) == 0)
-        {
-            fprintf(stderr, "Cannot find rgba visual\n");
-            return false;
-        }
-        rgbaFormat = XRenderFindVisualFormat(_dpy, rgbaVisual.visual);
-    }
-
-
-    Imlib_Image image = imlib_load_image(filename);
-    if (image == 0)
-        return false;
-
-    imlib_context_set_image(image);
-    int w = imlib_image_get_width();
-    int h = imlib_image_get_height();
-
-    if (width) *width = w;
-    if (height) *height = h;
-
-    // Making image with premultiplied alpha
-    Imlib_Image premul = imlib_create_image(w, h);
-    imlib_context_set_image(premul);
-    imlib_context_set_color(0, 0, 0, 255);
-    imlib_image_fill_rectangle(0, 0, w, h);
-    imlib_context_set_blend(1);
-    imlib_blend_image_onto_image(image, 0, 0, 0, w, h, 0, 0, w, h);
-    imlib_image_copy_alpha_to_image(image, 0, 0);
-
-    *pixmap = XCreatePixmap(_dpy, parent, w, h, 32);
-    imlib_context_set_display(_dpy);
-    imlib_context_set_colormap(colormap);
-    imlib_context_set_visual(rgbaVisual.visual);
-    imlib_context_set_drawable(*pixmap);
-    imlib_context_set_blend(0);
-    imlib_render_image_on_drawable(0, 0);
-
-    imlib_free_image();
-    imlib_context_set_image(image);
-    imlib_free_image();
-
-    *picture = XRenderCreatePicture(_dpy, *pixmap, rgbaFormat, 0, 0);
-
-    return true;
+    _bufferPicture = XRenderCreatePicture(_dpy, _bufferPix, XTools::xrenderFormat(), 0, 0);
 }
 
 
@@ -1150,7 +1001,7 @@ void TeleWindow::animate(Thumbnail *thumb, bool toSmall)
 {
         XFlush(_dpy);
 
-        
+
         XWindowAttributes attrs;
         XGetWindowAttributes(_dpy, thumb->clientWindow(), &attrs);
 
@@ -1226,3 +1077,10 @@ void TeleWindow::animate(Thumbnail *thumb, bool toSmall)
             //select(0, 0, 0, 0, &ts);
         }
 }
+
+
+
+void TeleWindow::showDesktop()
+{
+    XTools::showDesktop(Settings::instance()->showDesktopByIconify());
+}
